@@ -29,7 +29,7 @@ import("devel.git")
 import("net.fasturl")
 import("core.base.privilege")
 import("privilege.sudo")
-import("private.async.runjobs")
+import("async.runjobs")
 import("private.action.require.impl.environment")
 import("private.action.update.fetch_version")
 import("utils.archive")
